
#include <Definitions.hpp>

#if defined USE_SPEECH_ENGINES
// I include this header in CPP file, because of errors.
#include <C:/Program Files/Microsoft SDKs/Windows/v7.1/Samples/winui/speech/simpleaudio/simpleaudio_i.c>
#endif	// if defined USE_SPEECH_ENGINES

// The format we sample voice in.
static const WAVEFORMATEX g_VoiceSampleFormat =
{
	WAVE_FORMAT_PCM,		// wFormatTag
	1u,						// nChannels
	8000u,					// nSamplesPerSec
	16000u,					// nAvgBytesPerSec
	2u,						// nBlockAlign
	16u,					// wBitsPerSample
	sizeof (WAVEFORMATEX)	// cbSize
};
const bool BotChatter::Voice_RecordStart
(
	const char *const pUncompressedFile,
	const char *const pDecompressedFile,
	const char *const pMicInputFile
)
{
	if (!IsCodecInitialized ())
	{
		// Reliability checks.
		assert (m_pMicInputFileData == NULL);
		assert (m_pUncompressedFileData == NULL);
		assert (m_pDecompressedFileData == NULL);
		assert (m_bVoiceRecording == false);

		return false;
	}

	Voice_RecordStop ();

	GetCodec ()->ResetState ();

#if defined USE_SPEECH_ENGINES
	if (pMicInputFile == NULL)
	{
		// Create temp variable for easy TTS transformation :)
		static cvar_t *sp_TTS (NULL);
		if (sp_TTS == NULL)
		{
			static cvar_t s_TTS ("Voice_TTS", "Hello world!", FCVAR_SERVER);
			CVAR_REGISTER (&s_TTS);
			sp_TTS = CVAR_GET_POINTER (s_TTS.name);	// Get pointer from engine in case if we under metamod....
		}

	    // Initialize COM library on the current thread and identify the concurrency model as
		// single-thread apartment (STA). Applications must initialize the COM library before they
		// can call COM library functions other than CoGetMalloc and memory allocation functions.
		// New application developers may choose to use CoInitializeEx rather than CoInitialize, allowing
		// them to set the concurrency model to apartment or multi-threaded.
		HRESULT hr (CoInitialize (NULL));
		if (FAILED (hr))
		{
			(*g_engfuncs.pfnServerPrint) (va ("Error: Failed to initialize COM. (hr=0x%X)\n", hr));
			return false;
		}

//		CComPtr <ISpAudioPlug> pVoice;
		ISpVoice *pVoice;
		// Creates SAPI instance from COM
		hr = CoCreateInstance (CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);
		if(FAILED (hr))
		{
			CoUninitialize ();	// Uninitializes COM
			(*g_engfuncs.pfnServerPrint) (va ("Error: Failed to create SAPI voice instance. (hr=0x%X)\n", hr));
			return false;
		}
//		CComPtr <ISpAudioPlug> pSpStream;
		ISpAudioPlug *pSpStream;
		// Create basic SAPI stream object
		hr = CoCreateInstance (CLSID_SpAudioPlug, NULL, CLSCTX_ALL, IID_ISpAudioPlug, (void **)&pSpStream);
		if (FAILED (hr))
		{
			CoUninitialize ();	// Uninitializes COM
			(*g_engfuncs.pfnServerPrint) (va ("Error: Unable to create SAPI SpAudioPlug instance. (hr=0x%X)\n", hr));
			return false;
		}
		pSpStream->Init (VARIANT_TRUE, SAFT8kHz16BitMono/* Default HL engine voice format */);
		pVoice->SetOutput (pSpStream, FALSE);
		if (FAILED (hr))
		{
			CoUninitialize ();	// Uninitializes COM
			(*g_engfuncs.pfnServerPrint) (va ("Error: Unable to set SAPI voice output. (hr=0x%X)\n", hr));
			return false;
		}

		m_nMicInputFileBytes = 0u;
		m_pMicInputFileData = new char[MAX_WAVEFILEDATA_LEN];

		{
			char srRequest[64u];
			strncpy (srRequest, sp_TTS->string, sizeof (srRequest));
			const unsigned char iNumBytesPerSample (static_cast <unsigned char> (g_VoiceSampleFormat.wBitsPerSample / 8u));
			CRITICAL_SECTION csPlaybackQueue;
			InitializeCriticalSection (&csPlaybackQueue);

			// Convert the prompt to WCHAR* (that's what SAPI expects)
			size_t iLen = strlen (srRequest) + 1;
			wchar_t *wsPrompt = new wchar_t[iLen];
			mbstowcs (wsPrompt, srRequest, iLen);

			// Start the synthesis
			pVoice->Speak (wsPrompt, SPF_ASYNC, NULL);

			delete [] wsPrompt;	// Not needed anymore

			// Now loop until finished synthesizing
			SPVOICESTATUS svsStatus;
			VARIANT vData;
			short *pAudio;
			for (;;)
			{
				hr = pVoice->GetStatus (&svsStatus, NULL);
				if (FAILED (hr))
				{
					(*g_engfuncs.pfnServerPrint) (va ("Error: Failed to get status of ISpVoice! (hr=0x%X)\n", hr));
					break;
				}

				// check if there is some synthesized audio to send to the playback manager
				hr = pSpStream->GetData (&vData);
				if (hr == S_OK)	// NB: don't use SUCCEEDED here since GetData returns S_FALSE when no data is available, which is a success code
				{
					hr = SafeArrayAccessData (vData.parray, (void **)&pAudio);
					if (SUCCEEDED (hr))
					{
						int iBytes = vData.parray->rgsabound[0].cElements*vData.parray->cbElements;

						EnterCriticalSection (&csPlaybackQueue);
						{
//(*g_engfuncs.pfnServerPrint) (va ("Synthesis [%s]: Appending %u bytes to sample...\n", srRequest, iBytes));
		memcpy (m_pMicInputFileData + m_nMicInputFileBytes, pAudio, iBytes);
		m_nMicInputFileBytes += iBytes;
						}
						LeaveCriticalSection (&csPlaybackQueue);

						hr = SafeArrayUnaccessData (vData.parray);
					}
				}

				if (svsStatus.dwRunningState & SPRS_DONE)
					break;

				Sleep (10);
//			(*g_engfuncs.pfnServerPrint) (va ("Synthesis [%s] IS SLEEPING!!!\n", srRequest));
			}

			// Process potential left-over chunk of data
			hr = pSpStream->GetData (&vData);
			if (hr == S_OK)
			{
				hr = SafeArrayAccessData (vData.parray, (void **)&pAudio);
				if (SUCCEEDED (hr))
				{
					int iBytes = vData.parray->rgsabound[0].cElements*vData.parray->cbElements;

					EnterCriticalSection (&csPlaybackQueue);
					{
//(*g_engfuncs.pfnServerPrint) (va ("Synthesis [%s]: Appending %u bytes to sample... (POST)\n", srRequest, iBytes));
		memcpy (m_pMicInputFileData + m_nMicInputFileBytes, pAudio, iBytes);
		m_nMicInputFileBytes += iBytes;
					}
					LeaveCriticalSection (&csPlaybackQueue);

					hr = SafeArrayUnaccessData (vData.parray);
				}
			}

			DeleteCriticalSection (&csPlaybackQueue);

			// Display the synthesis request that is currently being processed
			(*g_engfuncs.pfnServerPrint) (va ("Synthesis finished for prompt [%s]\n", srRequest));
		}

		// Releases SAPI voice
		pVoice->Release ();

		CoUninitialize ();	// Uninitializes COM
	}
#endif	// if defined USE_SPEECH_ENGINES
	if (pMicInputFile != NULL)
	{
		char inputFilename[_MAX_PATH];
		_snprintf (inputFilename, sizeof (inputFilename), "%s/%s", g_gameModName, pMicInputFile);
		FILE *fp (fopen (inputFilename, "rb"));

		if (fp == NULL)
			return false;

		WaveSoundData wfp (fp);

		if (!wfp.IsValid ())
			return false;

		/// @todo 'm_pitch' VALUE IN THIS CLASS!!!!!!!!!!!!
		/// @todo INVERT PITCH VALUE!!!!!!!!!!!!
		const unsigned char botPitch (100u);
		const float pitch (botPitch * 0.01f);	// get playback pitch
		const unsigned short newSamplesPerSec (static_cast <unsigned short> (g_VoiceSampleFormat.nSamplesPerSec * pitch));

		// Reliability check.
		assert (newSamplesPerSec > 0u);

		// Check if we should convert input data....
		if ((wfp.GetHeader ().format.bitsPerSample != g_VoiceSampleFormat.wBitsPerSample || wfp.GetHeader ().format.sampleRate != newSamplesPerSec) && !wfp.ResampleInternal (newSamplesPerSec, static_cast <unsigned char> (g_VoiceSampleFormat.wBitsPerSample)))
			return false;
/*
		/// @note TEST OF MERGING TWO SAMPLES:
		{
			const char *const pMicInputFile2 ("voice_input_my_org.wav");
			char inputFilename2[_MAX_PATH];
			_snprintf (inputFilename2, sizeof (inputFilename2), "%s/%s", g_gameModName, pMicInputFile2);

			FILE *fp2 (fopen (inputFilename2, "rb"));

			if (fp2 == NULL)
				return false;

			WaveSoundData wfp2 (fp2);

			if (!wfp2.IsValid ())
				return false;

			// Check if we should convert input data....
			if ((wfp2.GetHeader ().format.bitsPerSample != g_VoiceSampleFormat.wBitsPerSample || wfp2.GetHeader ().format.sampleRate != newSamplesPerSec) && !wfp2.ResampleInternal (newSamplesPerSec, static_cast <unsigned char> (g_VoiceSampleFormat.wBitsPerSample)))
				return false;

			wfp += wfp2;
		}
*/
		// Apply volume to samples....
		/// @todo 'm_volume' VALUE IN THIS CLASS!!!!!!!!!!!!
		const unsigned char botVolume (255u);
		wfp.SetVolume (botVolume / 255.0f);

		m_nMicInputFileBytes = wfp.GetSamplesSize ();
		m_pMicInputFileData = new char[m_nMicInputFileBytes];
		memcpy (m_pMicInputFileData, wfp.GetSamples (), m_nMicInputFileBytes);
	}

	// Reliability check.
	assert (m_pMicInputFileData != NULL);

	if (pUncompressedFile != NULL)
	{
		m_pUncompressedFileData = new char[MAX_WAVEFILEDATA_LEN];
		m_nUncompressedDataBytes = 0;
		m_pUncompressedDataFilename = pUncompressedFile;
	}

	if (pDecompressedFile != NULL)
	{
		m_pDecompressedFileData = new char[MAX_WAVEFILEDATA_LEN];
		m_nDecompressedDataBytes = 0;
		m_pDecompressedDataFilename = pDecompressedFile;
	}

	m_CurMicInputFileByte = 0;
	m_MicStartTime = (*g_engfuncs.pfnTime) ();
	m_bVoiceRecording = true;

	return true;
}
void BotChatter::Voice_RecordStop (void)
{
	// Write the files out for debugging.
	if (m_pMicInputFileData != NULL)
	{
		delete [] m_pMicInputFileData;
		m_pMicInputFileData = NULL;
	}

	if (m_pUncompressedFileData != NULL)
	{
		WriteWaveFile (m_pUncompressedDataFilename, m_pUncompressedFileData, m_nUncompressedDataBytes, g_VoiceSampleFormat.wBitsPerSample, g_VoiceSampleFormat.nChannels, g_VoiceSampleFormat.nSamplesPerSec);
		delete [] m_pUncompressedFileData;
		m_pUncompressedFileData = NULL;
	}

	if (m_pDecompressedFileData != NULL)
	{
		WriteWaveFile (m_pDecompressedDataFilename, m_pDecompressedFileData, m_nDecompressedDataBytes, g_VoiceSampleFormat.wBitsPerSample, g_VoiceSampleFormat.nChannels, g_VoiceSampleFormat.nSamplesPerSec);
		delete [] m_pDecompressedFileData;
		m_pDecompressedFileData = NULL;
	}

	m_bVoiceRecording = false;
}
const unsigned short BotChatter::Voice_GetCompressedData (char *const pchDest, const int nCount, const bool bFinal)
{
	if (m_pMicInputFileData == NULL)
		return 0u;

	if (!IsCodecInitialized ())
		return 0u;

	short tempData[8192u];
	int gotten (0);
	int samplesNumber (0);

	// If they want to get the data from a file instead of the mic, use that.
	if (m_pMicInputFileData != NULL)
	{
		const double curtime ((*g_engfuncs.pfnTime) ());
		const int nShouldGet (static_cast <int> ((curtime - m_MicStartTime) * g_VoiceSampleFormat.nSamplesPerSec));

		gotten = min (sizeof (tempData) / BYTES_PER_SAMPLE, min (nShouldGet, (m_nMicInputFileBytes - m_CurMicInputFileByte) / BYTES_PER_SAMPLE));
		samplesNumber = gotten * BYTES_PER_SAMPLE;

		memcpy (tempData, m_pMicInputFileData + m_CurMicInputFileByte, samplesNumber);

		m_CurMicInputFileByte += samplesNumber;
		m_MicStartTime = curtime;
	}

	const unsigned short nCompressedBytes (static_cast <unsigned short> (GetCodec ()->Compress (reinterpret_cast <char *> (tempData), gotten, pchDest, nCount, bFinal)));

	// Write to our file buffers..
	if (m_pUncompressedFileData != NULL)
	{
		const int nToWrite (min (samplesNumber, MAX_WAVEFILEDATA_LEN - m_nUncompressedDataBytes));

		memcpy (m_pUncompressedFileData + m_nUncompressedDataBytes, tempData, nToWrite);

		m_nUncompressedDataBytes += nToWrite;
	}

	return nCompressedBytes;
}