
class BotChatter
{
	public:
		IVoiceCodec *m_encodeCodec;

		bool m_bVoiceRecording;	// Are we recording at the moment?

		// These are used for recording the wave data into files for debugging.
		#define MAX_WAVEFILEDATA_LEN	1024 * 1024
		char *m_pUncompressedFileData;
		int m_nUncompressedDataBytes;
		const char *m_pUncompressedDataFilename;

		char *m_pDecompressedFileData;
		int m_nDecompressedDataBytes;
		const char *m_pDecompressedDataFilename;

		char *m_pMicInputFileData;
		unsigned int m_nMicInputFileBytes;
		int m_CurMicInputFileByte;
		double m_MicStartTime;

	public:
		inline BotChatter (void) :
			m_encodeCodec (g_voiceCodecDLLWrapper->TryInitializeCodec (static_cast <unsigned char> (sv_voicequality->value))),

			m_bVoiceRecording (false),

			m_pUncompressedFileData (NULL),
			m_nUncompressedDataBytes (0),
			m_pUncompressedDataFilename (NULL),

			m_pDecompressedFileData (NULL),
			m_nDecompressedDataBytes (0),
			m_pDecompressedDataFilename (NULL),

			m_pMicInputFileData (NULL),
			m_nMicInputFileBytes (0u),
			m_CurMicInputFileByte (0),
			m_MicStartTime (0.0)
		{ /* VOID */ }
		inline ~BotChatter (void)
		{
			if (IsCodecInitialized ())
				ReleaseCodec ();

			delete [] m_pMicInputFileData;
			delete [] m_pUncompressedFileData;
			delete [] m_pDecompressedFileData;
		}

	//
	// Group: Functions.
	//
	public:
		inline       IVoiceCodec *const GetCodec           (void) const { return m_encodeCodec; }
		inline const bool               IsCodecInitialized (void) const { return GetCodec () != NULL && g_voiceCodecDLLWrapper->Initialized (); }
		inline       void               ReleaseCodec       (void)
		{
			// Reliability check.
			InternalAssert (IsCodecInitialized ());

			Voice_RecordStop ();

			// The commented code is used in the engine to resolve conflicts between
			// the codec and the class 'CDAudio' at the moment of unloading of the codec module
			// and also when calling the function 'IVoiceCodec::Release',
			// because both classes ('IVoiceCodec' and 'CDAudio') use the same library of "MSS" (Miles Sound System).
			// It is actual only for codec "voice_miles"!!!
			// However, I commented that code because I wanted to avoid wild hooks....
//			if (IS_DEDICATED_SERVER ())
				m_encodeCodec->Release ();
/*			else
			{
				CDAudio *const cdaudio (...);

				cdaudio->UNKNOWN2 ();

					m_encodeCodec->Release ();

				cdaudio->UNKNOWN1 ();
			}
*/
			m_encodeCodec = NULL;
		}
		inline void OnChangeCodec (const unsigned char quality)
		{
			// Reliability check....
			InternalAssert (!IsCodecInitialized ());

			m_encodeCodec = g_voiceCodecDLLWrapper->TryInitializeCodec (quality);
		}

		inline const bool Voice_IsRecording (void) const { return m_bVoiceRecording; }
		const bool Voice_RecordStart (const char *const uncompressedFile, const char *const decompressedFile, const char *const micInputFile);
		void Voice_RecordStop (void);
		const unsigned short Voice_GetCompressedData (char *const destination, const int count, const bool isFinal);
};