//========= Copyright  1996-2001, Valve LLC, All rights reserved. ============
//
// Purpose: 
//
// $NoKeywords: $
//=============================================================================

#include <Definitions.hpp>

static unsigned long ReadDWord(FILE *fp)
{
	unsigned long ret;
	fread(&ret, sizeof (ret), 1, fp);
	return ret;
}
static unsigned short ReadWord(FILE *fp)
{
	unsigned short ret;
	fread(&ret, sizeof (ret), 1, fp);
	return ret;
}
static void WriteDWord(FILE *fp, unsigned long val) 
{
	fwrite(&val, sizeof (val), 1, fp);
}
static void WriteWord(FILE *fp, unsigned short val) 
{
	fwrite(&val, sizeof (val), 1, fp);
}

bool ReadWaveFile
(
	const char *const pFilename,
	char *&pData,
	unsigned int &nDataBytes,
	unsigned short &wBitsPerSample,
	unsigned short &nChannels,
	unsigned int &nSamplesPerSec
)
{
	FILE *fp (fopen(pFilename, "rb"));
	if(fp == NULL)
		return false;

	unsigned char seekPosition (sizeof (char[GET_STATIC_STRING_LENGTH ("RIFF")]) + sizeof (unsigned long) + sizeof (char[GET_STATIC_STRING_LENGTH ("WAVE")]) + sizeof (char[GET_STATIC_STRING_LENGTH ("fmt ")]) + sizeof (unsigned long) + sizeof (unsigned short));

	fseek(fp, seekPosition, SEEK_SET);
	
	nChannels = ReadWord(fp);
	nSamplesPerSec = ReadDWord(fp);

	seekPosition += sizeof (nChannels) + sizeof (nSamplesPerSec) + sizeof (unsigned long) + sizeof (unsigned short);

	fseek(fp, seekPosition, SEEK_SET);
	wBitsPerSample = ReadWord(fp);

	seekPosition += sizeof (wBitsPerSample) + sizeof (char[GET_STATIC_STRING_LENGTH ("data")]);

	fseek(fp, seekPosition, SEEK_SET);
	nDataBytes = ReadDWord(fp);

	if (nDataBytes == 0u)
		return false;

	pData = new char[nDataBytes];

	if (pData == NULL)
	{
		fclose (fp);

		return false;
	}
/*! ORIGINAL CODE FROM HL1/SOURCE ENGINE
	seekPosition += sizeof (nDataBytes) + sizeof (unsigned long);	/// WHY WE SKIP FIRST FOUR BYTES OF DATA?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!??!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!?!??!?!?!

	fseek(fp, seekPosition, SEEK_SET);
*/
	fread(pData, nDataBytes, 1, fp);

	fclose (fp);

	return true;
}

bool WriteWaveFile(
	const char *const pFilename, 
	const char *const pData, 
	const unsigned int nDataBytes, 
	const unsigned short wBitsPerSample, 
	const unsigned short nChannels, 
	const unsigned int nSamplesPerSec)
{
	FILE *fp (fopen(pFilename, "wb"));
	if(fp == NULL)
		return false;

	// Write the RIFF chunk.
	fwrite("RIFF", GET_STATIC_STRING_LENGTH ("RIFF"), 1, fp);
	WriteDWord(fp, 0);
	fwrite("WAVE", GET_STATIC_STRING_LENGTH ("WAVE"), 1, fp);

	// Write the FORMAT chunk.
	fwrite("fmt ", GET_STATIC_STRING_LENGTH ("fmt "), 1, fp);
	
	WriteDWord(fp, 16);
	WriteWord(fp, WAVE_FORMAT_PCM);
	WriteWord(fp, nChannels);
	WriteDWord(fp, nSamplesPerSec);
	WriteDWord(fp, (unsigned long)((wBitsPerSample / 8) * nChannels * nSamplesPerSec));
	WriteWord(fp, (unsigned short)((wBitsPerSample / 8) * nChannels));
	WriteWord(fp, wBitsPerSample);

	// Write the DATA chunk.
	fwrite("data", GET_STATIC_STRING_LENGTH ("data"), 1, fp);
	WriteDWord(fp, nDataBytes);
	fwrite(pData, nDataBytes, 1, fp);

	// Go back and write the length of the riff file.
	const unsigned long dwVal (ftell(fp) - 8);
	fseek(fp, sizeof (unsigned long), SEEK_SET);
	WriteDWord(fp, dwVal);

	fclose(fp);
	return true;
}